@@ -66,6 +66,7 @@ typedef struct{
 	int result_front_junction_numbers[MAX_ALIGNMENT_PER_ANCHOR];
 	int all_back_alignments;
 	int all_front_alignments;
+	int known_junctions;
 
 //	unsigned int tmp_jump_length;
 //	unsigned int best_jump_length;
@@ -153,5 +154,6 @@ int is_funky_fragment(global_context_t * global_context, char * rname1, char * c
 
 void finalise_structural_variances(global_context_t * global_context);
 
-void get_event_two_coordinates(global_context_t * global_context, unsigned int event_no, char ** small_chro, unsigned int * small_pos, unsigned int * small_abs, char ** large_chro, unsigned int * large_pos, unsigned int * large_abs);
+void debug_show_event(global_context_t* global_context, chromosome_event_t * event);
+void get_event_two_coordinates(global_context_t * global_context, unsigned int event_no, char ** small_chro, int * small_pos, unsigned int * small_abs, char ** large_chro,  int * large_pos, unsigned int * large_abs);
 #endif
